function denoisedImg = CombiningLayers(denoisedLevels, noiseMap, sigma)

[Ny Nx Nc, numLayers] = size(denoisedLevels);

denoisedLevels = squeeze(reshape(denoisedLevels, 1, [], Nc, numLayers));
noiseMap = noiseMap(:);
Len = Nx * Ny;
denoisedImg = zeros(Len, Nc);

matSig = reshape(sigma, 1, []);
matSig = repmat(matSig, length(noiseMap), 1);
matMap = repmat(noiseMap, 1, length(sigma));

indP = length(sigma) - sum(matSig > matMap, 2);
indN = clamp(indP + 1, 1, numLayers);

im_dR = squeeze(denoisedLevels(:, 1, :));
im_dG = squeeze(denoisedLevels(:, 2, :));
im_dB = squeeze(denoisedLevels(:, 3, :));

subN = sub2ind(size(im_dR), 1 : size(im_dR,1), indN')';
subP = sub2ind(size(im_dR), 1 : size(im_dR,1), indP')';

alpha = repmat((noiseMap - sigma(indP)')./(sigma(indN)'-sigma(indP)'), 1, Nc);
alpha(indP == indN, :) = 1;
denoisedImg(:, 1) = alpha(:, 1) .* im_dR(subN) + (1-alpha(:, 1)) .* im_dR(subP);
denoisedImg(:, 2) = alpha(:, 2) .* im_dG(subN) + (1-alpha(:, 2)) .* im_dG(subP);
denoisedImg(:, 3) = alpha(:, 3) .* im_dB(subN) + (1-alpha(:, 3)) .* im_dB(subP);

denoisedImg = reshape(denoisedImg, Ny, Nx, []);