clear; clc;

%%% Parameters and paths ---------------------------

addpath('BM3D');
addpath('Functions');

% select a scenes by commenting it out
% sceneName = 'toasters_DOF_AL.pbrt';
% sceneName = 'chess_DOF.pbrt';
sceneName = 'poolball_MB.pbrt';

spp = '8';              % the number of samples per pixel

global g;
g = 3;                  % smoothness parameter, larger values results in
                        % more smoothness
%%% ------------------------------------------------

resultFolder = ['Results\', sceneName];
if (exist(resultFolder, 'dir'))
    rmdir(resultFolder, 's');
end

setenv('OMP_NUM_THREADS', '8');
current = pwd;
cd('bin&scenes\');

%%%% Adaptive Sampling -----------------------------
%%%% -----------------------------------------------
fprintf('Started adaptive sampling stage\n');
command = ['pbrt.exe ', sceneName, ' --budget ', spp];
system(command);
cd(current);
fprintf('Finished adaptive sampling stage\n\n\n');

%%%% Denoising -------------------------------------
%%%% -----------------------------------------------
fprintf('Started denoising stage\n');
fprintf('************************\n');

%%% Initialization and reading data-----------------
[img noiseMap maxNoise] = readData(sceneName);
%%% ------------------------------------------------

[ttime timeNoiseLevels finalImg] = Denoising(img, noiseMap, maxNoise);
fprintf('************************\n');
fprintf('Finished denoising stage\n');

%%% Writing outputs and timings ---
imwrite(mat2gray(noiseMap, [min(noiseMap(:)) max(noiseMap(:))]), sprintf('%s\\NoiseMap.bmp', resultFolder));
imwrite(uint8(finalImg), sprintf('%s\\DenoisedImage.bmp', resultFolder));

WriteTiming(resultFolder, timeNoiseLevels, ttime);
%%% -------------------------------

