// AdaptiveSampling.cpp : Defines the exported functions for the DLL application.
//

#include "stdafx.h"

#define ADAPTIVESAMPLING_EXPORTS

#include "AdaptiveSampling.h"
#include "Wavelet.h"
#include <math.h>
#include <Windows.h> 
#include <iostream>
#include <stdio.h>
#include <omp.h>
#include <string.h>
#include <vector>
#include <algorithm>
using namespace std;

ADAPTIVESAMPLING_API SamplerHandler* handler = 0;


#ifdef _MANAGED
#pragma managed(push, off)
#endif

void SamplerHandler::setParam(char* x_fn, int x_budget)
{
	char _tppath[400];
	char command[400];

	sprintf(_tppath, "..\\Results\\%s", x_fn);
	sprintf(command, "md %s", _tppath);

	system(command);

	m_path = new char[400];
	m_fn = new char[400];

	strcpy(m_path, _tppath);
	strcpy(m_fn, x_fn);

	m_budget = x_budget;
	m_maxNumSPP = 8 * x_budget;

	m_numIteration = 4;
	m_blockSize = new int[m_numIteration];
	m_averageSPP = new int[m_numIteration];

	m_stepBlockSize = 8;

	m_initSPP = 2;

	m_factor[0] = 7;
	m_factor[1] = 3;
	m_factor[2] = 1;
}
void SamplerHandler::SetScaleAndGamma(float x_scale, float x_gamma)
{
	m_scale = x_scale;
	m_gamma = x_gamma;
}

SamplerHandler::SamplerHandler()
{
    m_path = NULL;
}

SamplerHandler::~SamplerHandler()
{
  if(m_path)
  {
	  delete m_path;
	  m_path = NULL;
  }
  if(m_fn)
  {
	  delete m_fn;
	  m_fn = NULL;
  }
  
  if(m_blockSize)
  {
     delete m_blockSize; m_blockSize = NULL;
  }
  if(m_averageSPP)
  {
     delete m_averageSPP; m_averageSPP = NULL;
  }
  if(m_importanceMap)
  {
    delete m_importanceMap; m_importanceMap = NULL;
  }
  if(m_numMoreSamples)
  {
    delete m_numMoreSamples; m_numMoreSamples = NULL;
  }

  for(int i = 0; i < m_xDim * m_yDim; i++)
  {
	 m_finalColorR[i].clear();
	 m_finalColorG[i].clear();
	 m_finalColorB[i].clear();
  }

  delete m_finalColorR;
  delete m_finalColorG;
  delete m_finalColorB;
}


void SamplerHandler::allocateMemory(int x_xDim, int x_yDim, int x_SampleNumPerPixel)
{		
	m_xDim = x_xDim + 1;//important
	m_yDim = x_yDim + 1;//important

	int _size = m_xDim * m_yDim;
	
	m_budget *= _size; 

	m_numMoreSamples = new int[_size];
	memset(m_numMoreSamples,0,sizeof(int)*_size);
	
	int _maxBlockSize = 0;;
	for(int i = 0; i < m_numIteration; i++)
	  _maxBlockSize = max(_maxBlockSize, m_stepBlockSize);
	
	_maxBlockSize = max(_maxBlockSize,MINBLOCKSIZE);
	_maxBlockSize = max(_maxBlockSize, BLOCKNOISECALC);
	
	int _maxClusterSize = handler->m_maxNumSPP * _maxBlockSize * _maxBlockSize;
	int _maxNumPixel = _maxBlockSize * _maxBlockSize;

	for(int i = 0; i < 16; i++)
		mySampleClusters[i].initSampleCluster(_maxClusterSize, _maxNumPixel);
	
	m_finalColorR = new vf[_size];
	m_finalColorG = new vf[_size];
	m_finalColorB = new vf[_size];
}

void SamplerHandler::setInitialSamplingNum()
{
  int _size = m_xDim * m_yDim;
  for(int i = 0; i < _size; i++)
	  m_numMoreSamples[i] = m_initSPP;
  m_budgetStep = (m_budget - m_initSPP * m_xDim * m_yDim)/m_factor[0];
}


void SamplerHandler::addFinalColor(float x_r, float x_g, float x_b, int x_pind)
{
	
	m_finalColorR[x_pind].push_back(x_r);
	m_finalColorG[x_pind].push_back(x_g);
	m_finalColorB[x_pind].push_back(x_b);
}


void MakeDirectory(char* dirpath)
{
	char command[400];
	sprintf(command,"md %s",dirpath);
	system(command);
}

int SamplerHandler::getNumExistedSamples(int x_pind)
{

	if(x_pind >= m_xDim * m_yDim)
		printf("DEBUG::getNumExistedSamples::pind=%d\n",x_pind);
	int re =  (m_finalColorR[x_pind].size());
	return re;
}

float SamplerHandler::noiseMap(int x_it)
{
	myTimer callTimer;
	callTimer.startTimer();
		
	m_actualBlockSize = BLOCKNOISECALC;

	int _mapSize = m_xDim * m_yDim;

	
	float* _varMap = new float[_mapSize];
	float* _noiseMap = new float[_mapSize];

	# pragma omp parallel
	{
	#pragma omp for schedule(dynamic) nowait
		for(int by = 0; by < m_yDim; by++)	
			for(int bx = 0; bx < m_xDim; bx++)
			{
				int _indThread =  omp_get_thread_num();
				fillInTheCluster(x_it, _indThread, bx, by);
				
				int _indBlock = by * m_xDim + bx;
				
				float _tmpNoiseMap = noiseEstimation(mySampleClusters[_indThread]);
				float _tmpVarMap = varianceMap(_indBlock);
				
				_noiseMap[_indBlock] = sqrt(_tmpNoiseMap);
				_varMap[_indBlock] = sqrt(_tmpVarMap);
				
			}
	}
	
	m_maxNoise = -1 * FLT_MAX;

	int _sizeY = m_yDim/2;
	int _sizeX = m_xDim/2;
	# pragma omp parallel
	{
	#pragma omp for schedule(dynamic) nowait
		for(int by = 0; by < _sizeY; by++)	
			for(int bx = 0; bx < _sizeX; bx++)
			{
				float _tmpNoiseMap = 0;
				int _indPixel;
				for (int i = 0; i < 2; i++)
				{
					for (int j = 0; j < 2; j++)
					{
						_indPixel = min(2 * by + i, m_yDim) * m_xDim + min(2 * bx + j, m_xDim);
						_tmpNoiseMap += _noiseMap[_indPixel];
					}
				}
				_tmpNoiseMap /= 4;

				m_maxNoise = max(m_maxNoise, _tmpNoiseMap);

				for (int i = 0; i < 2; i++)
				{
					for (int j = 0; j < 2; j++)
					{
						_indPixel = min(2 * by + i, m_yDim) * m_xDim + min(2 * bx + j, m_xDim);
						_noiseMap[_indPixel] = _tmpNoiseMap;
					}
				}
			}
	}
	
	

	_noiseMap = dilation(_noiseMap, m_xDim, m_yDim);
	_varMap = dilation(_varMap, m_xDim, m_yDim);
	
	float _maxNoiseMap = -1 * FLT_MAX;
	# pragma omp parallel
		{
	#pragma omp for schedule(dynamic) nowait
		for(int by = 0; by < m_yDim; by++)	
			for(int bx = 0; bx < m_xDim; bx++)
			{
				int _pos = by * m_xDim + bx;
				_noiseMap[_pos] *= powf(_varMap[_pos], 0.25f);
				_maxNoiseMap = max(_maxNoiseMap, _noiseMap[_pos]);
			}
		}

	float _invMaxNoiseMap = 1/_maxNoiseMap;
	# pragma omp parallel
		{
	#pragma omp for schedule(dynamic) nowait
			for(int by = 0; by < m_yDim; by++)	
				for(int bx = 0; bx < m_xDim; bx++)
				{
					int _pos = by * m_xDim + bx;
					_noiseMap[_pos] *= _invMaxNoiseMap * m_maxNoise;
				}
		}

				
	callTimer.stopTimer();
	
	char tpbuf[300];
	sprintf(tpbuf, "%s\\NoiseMap.txt", m_path);
	FILE *output = fopen(tpbuf, "wb");
	
	fwrite(&m_maxNoise, 4, 1, output);
	for (int i = 0; i < _mapSize; i++)
	{
		fwrite(&_noiseMap[i], 4, 1, output);
	}

	fclose(output);
	
	if(_noiseMap)
	{
		delete _noiseMap;
		_noiseMap = NULL;
	}
	
	return((float)(callTimer.getTime()));
}

void SamplerHandler::calImportanceMap(int x_it)
{
	m_maxNoise = -1 * FLT_MAX;	
	m_actualBlockSize = max(m_stepBlockSize, MINBLOCKSIZE);

	getImportanceMapSize(x_it);

	int _mapSize = m_importanceMapWidth * m_importanceMapHeight;
	m_importanceMap = new float[_mapSize];
	memset(m_importanceMap, 0, sizeof(float) * _mapSize);

	# pragma omp parallel
	{
	#pragma omp for schedule(dynamic) nowait
	for(int by = 0; by < m_importanceMapHeight; by++)	
	  for(int bx = 0; bx < m_importanceMapWidth; bx++)
	  {
		  int _indThread =  omp_get_thread_num();
		  fillInTheCluster(x_it, _indThread, bx, by);
		  int _indBlock = by * m_importanceMapWidth + bx;
		  m_importanceMap[_indBlock] = calImportanceForThisCluster(mySampleClusters[_indThread]);
	  }
	}

	m_importanceMap = dilation(m_importanceMap, m_importanceMapWidth, m_importanceMapHeight);
	
	int psize = m_xDim * m_yDim;
	memset(m_numMoreSamples, 0, sizeof(int) * psize);//necessary

	// now we have the importance map, we need to calculate how many more samples need to be thrown to each
	// pixel at this iteration. Remind that the final spp should not exceed MAXSAMPLEPERPIXEL.
	addSample2ThisBlock(x_it);

	if(m_importanceMap)
	{
		delete m_importanceMap;
		m_importanceMap = NULL;
	}

	if (m_stepBlockSize > 2)
		m_stepBlockSize /= 2;
}

void SamplerHandler::fillInTheCluster(int x_it, int x_indThread, int x_bx, int x_by)
{
	mySampleClusters[x_indThread].byind = x_by;
	mySampleClusters[x_indThread].bxind = x_bx;

	// move data into the 'indThread'-th sample cluster
	int pymin, pymax, pxmin, pxmax;
	getClusterIndices(x_it, pymin, pymax, pxmin, pxmax, x_bx, x_by);

	int _tpsInd = 0;
	int _fColorInd = 0;
	
	memset(mySampleClusters[x_indThread].m_finalColorR, 0, sizeof(float) * m_actualBlockSize * m_actualBlockSize);
	memset(mySampleClusters[x_indThread].m_finalColorG, 0, sizeof(float) * m_actualBlockSize * m_actualBlockSize);
	memset(mySampleClusters[x_indThread].m_finalColorB, 0, sizeof(float) * m_actualBlockSize * m_actualBlockSize);

	for(int py = pymin; py <= pymax; py++)
		for(int px = pxmin; px <= pxmax; px++)
		{
			int _indPixel = py * m_xDim + px;
			if (py >= 0 && py < m_yDim && px >= 0 && px < m_xDim)
			{
				int snum = getNumExistedSamples(_indPixel);
				for(int s = 0; s < snum; s++)
				{
					mySampleClusters[x_indThread].sR[_tpsInd] = m_finalColorR[_indPixel][s];
					mySampleClusters[x_indThread].sG[_tpsInd] = m_finalColorG[_indPixel][s];
					mySampleClusters[x_indThread].sB[_tpsInd] = m_finalColorB[_indPixel][s];
					mySampleClusters[x_indThread].m_finalColorR[_fColorInd] += (m_finalColorR[_indPixel][s]);
					mySampleClusters[x_indThread].m_finalColorG[_fColorInd] += (m_finalColorG[_indPixel][s]);
					mySampleClusters[x_indThread].m_finalColorB[_fColorInd] += (m_finalColorB[_indPixel][s]);
					_tpsInd++;
				}
				mySampleClusters[x_indThread].m_finalColorR[_fColorInd] /= snum;
				mySampleClusters[x_indThread].m_finalColorG[_fColorInd] /= snum;
				mySampleClusters[x_indThread].m_finalColorB[_fColorInd] /= snum;
				mySampleClusters[x_indThread].m_finalColorR[_fColorInd] = quantize(mySampleClusters[x_indThread].m_finalColorR[_fColorInd]);
				mySampleClusters[x_indThread].m_finalColorG[_fColorInd] = quantize(mySampleClusters[x_indThread].m_finalColorG[_fColorInd]);
				mySampleClusters[x_indThread].m_finalColorB[_fColorInd] = quantize(mySampleClusters[x_indThread].m_finalColorB[_fColorInd]);
				
			}
			else
			{
				mySampleClusters[x_indThread].m_finalColorR[_fColorInd] = 0;
				mySampleClusters[x_indThread].m_finalColorG[_fColorInd] = 0;
				mySampleClusters[x_indThread].m_finalColorB[_fColorInd] = 0;	
			}
			_fColorInd++;
		}
		mySampleClusters[x_indThread].valid_sample_size = _tpsInd;
}

void SamplerHandler::getClusterIndices(int x_it, int &x_pymin, int &x_pymax, int &x_pxmin, int &x_pxmax, int x_bx, int x_by)
{
	if(x_it == m_numIteration)
	{
		x_pymin = x_by - (m_actualBlockSize) / 2;
		x_pymax = x_by + (m_actualBlockSize) / 2 - 1;
		x_pxmin = x_bx - (m_actualBlockSize) / 2;
		x_pxmax = x_bx + (m_actualBlockSize) / 2 - 1;
	}
	else
	{
		x_pymin = x_by * m_stepBlockSize - (m_actualBlockSize-m_stepBlockSize)/2;
		x_pxmin = x_bx * m_stepBlockSize - (m_actualBlockSize-m_stepBlockSize)/2;
		x_pymax = x_by * m_stepBlockSize + m_stepBlockSize - 1 + (m_actualBlockSize-m_stepBlockSize)/2;
		x_pxmax = x_bx * m_stepBlockSize + m_stepBlockSize - 1 + (m_actualBlockSize-m_stepBlockSize)/2;
	}
}

void SamplerHandler::getImportanceMapSize(int x_it)
{
	m_importanceMapWidth = ceil(float(m_xDim) / m_stepBlockSize);
	m_importanceMapHeight = ceil(float(m_yDim) / m_stepBlockSize);

	if((m_importanceMapWidth * m_stepBlockSize == m_xDim) && (m_importanceMapHeight * m_stepBlockSize == m_yDim))
		m_isAppended = false;
	else
		m_isAppended = true;
}
	 
void SamplerHandler::addSample2ThisBlock(int x_it)
{
	double _sum = 0;
	float _max = -1*FLT_MAX;
	for(int i = 0; i < m_importanceMapHeight * m_importanceMapWidth; i++)
		_sum += m_importanceMap[i];

	int _size = m_xDim * m_yDim;
	int _budget = m_budgetStep;

	int pymin, pymax, pxmin, pxmax;
	for(int by = 0; by < m_importanceMapHeight; by++)	
		for(int bx = 0; bx < m_importanceMapWidth; bx++)
		{
			if(by == m_importanceMapHeight - 1)
			{
				by = m_importanceMapHeight - 1;
			}
			int _indBlock = bx + by * m_importanceMapWidth;
			pymin = by * m_stepBlockSize;
			pymax = (by + 1) * m_stepBlockSize - 1;
			pxmin = bx * m_stepBlockSize;
			pxmax = (bx + 1) * m_stepBlockSize - 1;
			pymin = max(pymin, 0);
			pxmin = max(pxmin, 0);
			pymax = min(pymax, m_yDim - 1);
			pxmax = min(pxmax, m_xDim - 1);
			int _numMoreSamples = 0;
			int _numPixelsInThisBlock = (pymax - pymin + 1) * (pxmax - pxmin + 1);
			if (_sum != 0)
				_numMoreSamples = (m_importanceMap[_indBlock]/_sum * _budget + 0.5) / _numPixelsInThisBlock;

			_sum -= m_importanceMap[_indBlock];
			for(int py=pymin;py<=pymax;py++)
				for(int px=pxmin;px<=pxmax;px++)
				{
					int _indPixel = px + py * m_xDim;
					int _diffNumSamp = handler->m_maxNumSPP - getNumExistedSamples(_indPixel);
					if (_diffNumSamp > _numMoreSamples)
					{
						_budget -= _numMoreSamples;
						m_numMoreSamples[_indPixel] +=_numMoreSamples;
					}
					else
					{
						_budget -= _diffNumSamp;
						m_numMoreSamples[_indPixel] += _diffNumSamp;
					}
				}
		}
		m_snum = 0;
		for (int i = 0; i < m_xDim * m_yDim; i++)
			m_snum += getNumExistedSamples(i) + m_numMoreSamples[i];

		m_budgetStep = (m_budget - m_snum) / m_factor[x_it];
}



float SamplerHandler::calImportanceForThisCluster(SampleCluster &mSampleCluster)
{
	float _varMap = contrastMap(mSampleCluster);
	float _noise = noiseEstimation(mSampleCluster);
	m_maxNoise = max(m_maxNoise, _noise);
	return sqrt(_varMap * _noise);
}

float SamplerHandler::contrastMap(SampleCluster &mSampleCluster)
{
	float eeps = 1e-5;
	float w_mdas_R = 0;
	float w_mdas_G = 0;
	float w_mdas_B = 0;
	float mean_R = 0;
	float mean_G = 0;
	float mean_B = 0;

	for(int i=0;i<mSampleCluster.valid_sample_size;i++)
	{
		mean_R += mSampleCluster.sR[i];
		mean_G += mSampleCluster.sG[i];
		mean_B += mSampleCluster.sB[i];
	}
	mean_R /= mSampleCluster.valid_sample_size;
	mean_G /= mSampleCluster.valid_sample_size;
	mean_B /= mSampleCluster.valid_sample_size;

	if(mean_R>eeps)
	{
		for(int i=0;i<mSampleCluster.valid_sample_size;i++)
			w_mdas_R += abs(mSampleCluster.sR[i]-mean_R);
		w_mdas_R /= (mean_R*mSampleCluster.valid_sample_size);
	}
	else
	{
		w_mdas_R = 0;
	}

	if(mean_G>eeps)
	{
		for(int i=0;i<mSampleCluster.valid_sample_size;i++)
			w_mdas_G += abs(mSampleCluster.sG[i]-mean_G);
		w_mdas_G /= (mean_G*mSampleCluster.valid_sample_size);
	}
	else
	{
		w_mdas_G = 0;
	}

	if(mean_B>eeps)
	{
		for(int i=0;i<mSampleCluster.valid_sample_size;i++)
			w_mdas_B += abs(mSampleCluster.sB[i]-mean_B);
		w_mdas_B /= (mean_B*mSampleCluster.valid_sample_size);
	}
	else
	{
		w_mdas_B = 0;
	}

	return (w_mdas_R+w_mdas_G+w_mdas_B)/3.0;
};

float SamplerHandler::varianceMap(int x_ind)
{
	float eeps = 1e-5;
	float _varR = 0;
	float _varG = 0;
	float _varB = 0;
	float mean_R = 0;
	float mean_G = 0;
	float mean_B = 0;
	
	int _num = getNumExistedSamples(x_ind);
	for(int i = 0; i < _num; i++)
	{
		mean_R += quantize(m_finalColorR[x_ind][i]);
		mean_G += quantize(m_finalColorG[x_ind][i]);
		mean_B += quantize(m_finalColorB[x_ind][i]);
	}
	mean_R /= _num;
	mean_G /= _num;
	mean_B /= _num;

	if(mean_R>eeps)
	{
		for(int i=0;i<_num;i++)
			_varR += (quantize(m_finalColorR[x_ind][i])-mean_R)*(quantize(m_finalColorR[x_ind][i])-mean_R);
		_varR /= (powf(mean_R,2)*_num);
	}
	else
	{
		_varR = 0;
	}

	if(mean_G>eeps)
	{
		for(int i=0;i<_num;i++)
			_varG += (quantize(m_finalColorG[x_ind][i])-mean_G)*(quantize(m_finalColorG[x_ind][i])-mean_G);
		_varG /= (powf(mean_G,2)*_num);
	}
	else
	{
		_varG = 0;
	}

	if(mean_B>eeps)
	{
		for(int i=0;i<_num;i++)
			_varB += (quantize(m_finalColorB[x_ind][i])-mean_B)*(quantize(m_finalColorB[x_ind][i])-mean_B);
		_varB /= (powf(mean_B,2)*_num);
	}
	else
	{
		_varB = 0;
	}

	return (_varR + _varG + _varB)/3.0;
};

float SamplerHandler::noiseEstimation(SampleCluster &mSampleCluster)
{
	int _length = m_actualBlockSize * m_actualBlockSize;
	float _factor = 1 / 0.6745;

	wavelet_transfer_2d(m_actualBlockSize, m_actualBlockSize, mSampleCluster.m_finalColorR, 3, 1);
	float *_diagWaveCoeffR = getDiagDetail(mSampleCluster.m_finalColorR, m_actualBlockSize);
	float _noiseR = median(_diagWaveCoeffR, m_actualBlockSize) * _factor;

	wavelet_transfer_2d(m_actualBlockSize, m_actualBlockSize, mSampleCluster.m_finalColorG, 3, 1);
	float *_diagWaveCoeffG = getDiagDetail(mSampleCluster.m_finalColorG, m_actualBlockSize);
	float _noiseG = median(_diagWaveCoeffG, m_actualBlockSize) * _factor;

	wavelet_transfer_2d(m_actualBlockSize, m_actualBlockSize, mSampleCluster.m_finalColorB, 3, 1);
	float *_diagWaveCoeffB = getDiagDetail(mSampleCluster.m_finalColorB, m_actualBlockSize);
	float _noiseB = median(_diagWaveCoeffB, m_actualBlockSize) * _factor;

	return (powf(_noiseR,2) + powf(_noiseG,2) + powf(_noiseB,2))/3;
};



float* SamplerHandler::dilation(float *x_input, int x_w, int x_h)
{
	float* _output = new float[x_h*x_w];
	for (int yy = 0; yy<x_h; yy++)
		for(int xx = 0; xx<x_w; xx++)
		{
			// compare the N8 neighbors
			int pind = yy*x_w+xx;
			_output[pind] = x_input[pind]; //important
			int ImaxInd = pind;
			float Imax = x_input[ImaxInd];
			int nind;			  
			// left-up
			nind = max(0,yy-1) * x_w + max(0,xx-1);
			if(x_input[nind]>Imax)
				Imax = x_input[nind];
			// middle-up
			nind = max(0,yy-1) * x_w + xx;
			if(x_input[nind]>Imax)
				Imax = x_input[nind];
			// right-up
			nind = max(0,yy-1) * x_w + min(x_w-1,xx+1);
			if(x_input[nind]>Imax)
				Imax = x_input[nind];
			// right-middle
			nind = yy * x_w + min(x_w-1,xx+1);
			if(x_input[nind]>Imax)
				Imax = x_input[nind];
			// right-bottom
			nind = min(x_h-1,yy+1) * x_w + min(x_w-1,xx+1);
			if(x_input[nind]>Imax)
				Imax = x_input[nind];
			// middle-bottom
			nind = min(x_h-1,yy+1) * x_w + xx;
			if(x_input[nind]>Imax)
				Imax = x_input[nind];
			// left-bottom
			nind = min(x_h-1,yy+1) * x_w + max(0,xx-1);
			if(x_input[nind]>Imax)
				Imax = x_input[nind];
			// left-middle
			nind = yy * x_w + max(0,xx-1);
			if(x_input[nind]>Imax)
				Imax = x_input[nind];

			// replace with Imax
			if(Imax>x_input[pind])
				_output[pind] = Imax;			  
		}

	delete x_input;
	x_input = NULL;
	return _output;	
}

void SamplerHandler::imWriteImportanceMap()
{
	float r_max = -1.0*FLT_MAX;

	for(int i=0;i<m_importanceMapHeight*m_importanceMapWidth;i++)
		r_max = max(r_max,m_importanceMap[i]);

	for(int i=0;i<m_importanceMapHeight*m_importanceMapWidth;i++)
		m_importanceMap[i] = (m_importanceMap[i])/r_max;

	// show the importance map
	int tpsize = m_stepBlockSize;
	Image CMimg(m_xDim,m_yDim);
	for(int by=0;by<m_importanceMapHeight;by++)	
		for(int bx=0;bx<m_importanceMapWidth;bx++)
		{
			int indBlock = by*m_importanceMapWidth+bx;
			int pxmin,pxmax,pymin,pymax;
			pymin = by*tpsize;
			pxmin = bx*tpsize;
			if(m_isAppended)
			{
				if(by==m_importanceMapHeight-1)
					pymax = m_yDim-1;
				else
					pymax = by*tpsize+tpsize-1;
				if(bx==m_importanceMapWidth-1)
					pxmax = m_xDim-1;
				else
					pxmax = bx*tpsize+tpsize-1;  
			}
			else
			{
				pymax = by*tpsize+tpsize-1;
				pxmax = bx*tpsize+tpsize-1;
			}

			for(int py=pymin;py<=pymax;py++)
				for(int px=pxmin;px<=pxmax;px++)
				{
					int indPixel = py*m_xDim+px;
					*(CMimg.data + indPixel*3) = (u08) 255.0*m_importanceMap[indBlock];
					*(CMimg.data + indPixel*3+1) = (u08) 255.0*m_importanceMap[indBlock];
					*(CMimg.data + indPixel*3+2) = (u08) 255.0*m_importanceMap[indBlock];
				}
		}
	char tpbuf[300];
	sprintf(tpbuf, "%s\\ImportanceMap.bmp", m_path);

	CMimg.saveImage(tpbuf);
}

void SamplerHandler::imWriteAdaptiveSampledImage()
{
	Image mImg(m_xDim,m_yDim);
	for(int i=0;i<m_yDim;i++)
		for(int j=0;j<m_xDim;j++)
		{			
			float tpcR = 0;
			float tpcG = 0;
			float tpcB = 0;
			int indPixel = i*m_xDim+j;
			int actualNumSampleInThisPixel = getNumExistedSamples(indPixel);
			for(int s=0;s<actualNumSampleInThisPixel;s++)
			{
				tpcR += m_finalColorR[indPixel][s];
				tpcG += m_finalColorG[indPixel][s];
				tpcB += m_finalColorB[indPixel][s];
			}
			
			*(mImg.data+(i*m_xDim+j)*3  ) = (u08)(255.0*max(0,min(1,powf(tpcR/actualNumSampleInThisPixel/m_scale,m_gamma)/0.9))+0.5);
			*(mImg.data+(i*m_xDim+j)*3+1) = (u08)(255.0*max(0,min(1,powf(tpcG/actualNumSampleInThisPixel/m_scale,m_gamma)/0.9))+0.5);
			*(mImg.data+(i*m_xDim+j)*3+2) = (u08)(255.0*max(0,min(1,powf(tpcB/actualNumSampleInThisPixel/m_scale,m_gamma)/0.9))+0.5);
		}
		char buf[300];

		sprintf(buf, "%s\\AdaptiveSampledImage-%.1fspp.bmp", m_path, float(m_snum)/(m_xDim*m_yDim));
		mImg.saveImage(buf);	
}

void SamplerHandler::imWriteSampleDistribution()
{
	int numPixel = m_xDim*m_yDim;
	float* TPnumSampleofThisPixel = new float[numPixel]; 
	memset(TPnumSampleofThisPixel,0,numPixel*sizeof(float));

	for(int i=0;i<numPixel;i++)
		TPnumSampleofThisPixel[i] = getNumExistedSamples(i);

	normalize2Dsignal(TPnumSampleofThisPixel,numPixel);

	char buf[300];
	sprintf(buf, "%s\\SampleDistribution.bmp", m_path);
	imshow(TPnumSampleofThisPixel,TPnumSampleofThisPixel,TPnumSampleofThisPixel,m_xDim,m_yDim,1,buf);
	delete TPnumSampleofThisPixel;
	TPnumSampleofThisPixel = NULL;
}

void SamplerHandler::imshow(float* r,float* g,float* b, int m_xDim, int m_yDim, int sDim,char* img_name)
{
	float r_max = -1.0*FLT_MAX;
	float r_min = FLT_MAX;
	float g_max = -1.0*FLT_MAX;
	float g_min = FLT_MAX;
	float b_max = -1.0*FLT_MAX;
	float b_min = FLT_MAX;
	int size = m_xDim*m_yDim*sDim;
	for(int i=0;i<size;i++)
	{
		r_max = max(r_max,r[i]);
		r_min = min(r_min,r[i]);
		g_max = max(g_max,g[i]);
		g_min = min(g_min,g[i]);
		b_max = max(b_max,b[i]);
		b_min = min(b_min,b[i]);
	}
	for(int i=0;i<size;i++)
	{
		r[i] = (r[i]-r_min)/(r_max-r_min);
		g[i] = (g[i]-g_min)/(g_max-g_min);
		b[i] = (b[i]-b_min)/(b_max-b_min);
	}
	Image mImg(m_xDim,m_yDim);
	for(int i=0;i<m_yDim;i++)
		for(int j=0;j<m_xDim;j++)
		{
			float tpcR = 0;
			float tpcG = 0;
			float tpcB = 0;
			for(int s=0;s<sDim;s++)
			{
				int ind = (i*m_xDim+j)*sDim+s;
				tpcR += r[ind]/sDim; 
				tpcG += g[ind]/sDim; 
				tpcB += b[ind]/sDim; 
			}
			*(mImg.data+(i*m_xDim+j)*3  ) = (u08)(255.0*tpcR);
			*(mImg.data+(i*m_xDim+j)*3+1) = (u08)(255.0*tpcG);
			*(mImg.data+(i*m_xDim+j)*3+2) = (u08)(255.0*tpcB);
		}
		mImg.saveImage(img_name);
}



void SamplerHandler::normalize2Dsignal(float* signal,int size)
{
	float r_max = -1.0*FLT_MAX;
	float r_min = FLT_MAX;

	for(int i=0;i<size;i++)
	{
		r_max = max(r_max,signal[i]);
		r_min = min(r_min,signal[i]);
	}

	for(int i=0;i<size;i++)
		signal[i] = (signal[i]-r_min)/(r_max-r_min);
}

float SamplerHandler::median(float *x_data, int x_blkSize)
{
	int _size = x_blkSize * x_blkSize * .25;
	for (int i = 0; i < _size; i++)
	{
		x_data[i] = fabs(x_data[i]);
	}
	std::sort (x_data, x_data+_size);
	return (fabs(x_data[int(_size*.5)])+fabs(x_data[int(_size*.5)-1]))/2;
}

float SamplerHandler::quantize(float value)
{
	value = max(0, min(1.f, powf(value/m_scale,m_gamma)/0.9));
	value = int((value)*255+0.5);
	return value;
}


ADAPTIVESAMPLING_API void setSamplerParam(char* x_fn, int x_budget)
{
	handler = new SamplerHandler();
	handler->setParam(x_fn, x_budget);

}

#ifdef _MANAGED
#pragma managed(pop)
#endif

