#ifndef LEISAMPLER_H
#define LEISAMPLER_H

#ifdef ADAPTIVESAMPLING_EXPORTS
#define ADAPTIVESAMPLING_API __declspec(dllexport)
#else
#define ADAPTIVESAMPLING_API __declspec(dllimport)
#endif
ADAPTIVESAMPLING_API void setSamplerParam(char* x_fn, int x_budget);

#include "Timer.h"
#include "SampleCluster.h"
#include <memory.h>
#include "Image.h"
#include "globals.h"
#include <vector>
using namespace std;


typedef vector<float> vf;

class ADAPTIVESAMPLING_API SamplerHandler
{
public:
	//---
	myTimer timer;
	char* m_path;
    char* m_fn;
	int m_maxNumSPP;

	int m_xDim;
	int m_yDim;

	vf* m_finalColorR;
	vf* m_finalColorG;
	vf* m_finalColorB;

	int m_numIteration;
	int* m_blockSize; // size of the block that I use to compute importance map at this iteration
	int* m_averageSPP; // average number of samples per pixel at this iteration
	float* m_importanceMap;// importance map for the whole image, it will be updated at the beginning of each iteration of adaptive sampling
	int m_importanceMapHeight;
	int m_importanceMapWidth;
	int m_stepBlockSize;
	int m_actualBlockSize;
	float m_maxNoise;

	int m_budget;
	int m_budgetStep;
	int m_initSPP;
	float m_factor[3];
	int m_snum;

	float m_scale;
	float m_gamma;
	
	bool m_isAppended;// flag to show if the image size is exactly the integer times the BlockSize

	int* m_numMoreSamples; // how many samples need to be added for each pixel at each iteration
	SampleCluster mySampleClusters[16];

	SamplerHandler();
    ~SamplerHandler();
	void setParam(char* x_fn, int x_budget);
	void SetScaleAndGamma(float x_scale, float x_gamma);
	void allocateMemory(int x_xDim, int x_yDim, int x_SampleNumPerPixel);
	void addFinalColor(float x_r, float x_g, float x_b, int x_pind);
	void calImportanceMap(int x_it);
	float calImportanceForThisCluster(SampleCluster &mSampleCluster);
	void getImportanceMapSize(int x_it);
	void getClusterIndices(int x_it, int &x_pymin, int &x_pymax, int &x_pxmin, int &x_pxmax, int x_bx, int x_by);
	void fillInTheCluster(int x_it, int x_indThread, int x_bx, int x_by);
	float contrastMap(SampleCluster &mSampleCluster);
	float varianceMap(int x_ind);
	float noiseEstimation(SampleCluster &mSampleCluster);
	float* dilation(float* x_input, int x_w, int x_h);
	float noiseMap(int x_it);
	void imWriteImportanceMap();
	void addSample2ThisBlock(int x_it);
	void imWriteAdaptiveSampledImage();
	void imWriteSampleDistribution();
	void normalize2Dsignal(float* signal, int size);
	void imshow(float* r,float* g,float* b, int xDim, int yDim, int sDim, char* img_name);
	int getNumExistedSamples(int pind);
	void setInitialSamplingNum();
	float median(float *x_data, int x_blkSize);
	float quantize(float value);
};

extern ADAPTIVESAMPLING_API SamplerHandler* handler;


#endif