#include "stdafx.h"

#include <assert.h>
#include <stdio.h>
#include <math.h>

#include "Image.h"

/* constructor which loads up the image given a filename */
Image :: Image(char *name) {
	CImage image;
	data = NULL;
	width = 0;
	height = 0;

	/* load the image using the windows class */
	image.Load(name);

	/* do checks */
	basicImageChecks(&image);

	/* now copy the image data to the data structure */
	copyImageData(&image);

	return;
}

Image :: Image(int imgWidth, int imgHeight) {
	CImage image;
	
	width = imgWidth;
	height = imgHeight;
	data = (u08 *)malloc(sizeof(u08) * imgWidth * imgHeight * 3);
	for(int i=0; i<imgHeight; i++)
		for(int j=0; j<imgWidth; j++)
		{
			
		 *(data + (i * imgWidth + j)*3) = 0;
		 *(data + (i * imgWidth + j)*3 + 1) = 0;
		 *(data + (i * imgWidth + j)*3 + 2) = 0;
		   

		}



	return;
}


/* make sure the image was read in correctly and that it is full color (24bpp) */
void Image :: basicImageChecks(CImage *image) {
	if (image->IsNull()) {
		fprintf(stderr,"ERROR! The given source image was not loaded properly\n");
		fflush(stderr);
		exit(-1);
	}

	if(image->GetBPP() != 24) {
		fprintf(stderr,"ERROR! The given source image does not have 24 bpp\n");
		fflush(stderr);
		exit(-1);
	}

	return;
}

/* copies the image data from the CImage class into a unsigned char data array */
void Image :: copyImageData(CImage *image) {
	int i, j;
	
	/* copy dimensions */
	width = image->GetWidth();
	height = image->GetHeight();

	/* make room for the data array */
	data = (u08 *)malloc(sizeof(u08) * width * height * 3);

	/* now copy the contents from the CImage array to the unsigned char data array */
	int pitch = image->GetPitch();
	u08 *ptr_src = (u08 *)image->GetBits();
	u08 *ptr_dst = data;

	for (j = 0; j < height; j++) {
		for (i = 0; i < width; i++) {
			*(ptr_dst++) = *(ptr_src + 2);
			*(ptr_dst++) = *(ptr_src + 1);
			*(ptr_dst++) = *(ptr_src);

			ptr_src += 3;
		}
		ptr_src += pitch - (width * 3); 
	}

	return;
}


/* copies the image data from the unsigned char data array back into the  CImage class for saving the image out */
void Image :: copyBackImageData(CImage *image){
	int i, j;
	int pitch = image->GetPitch();
	u08 *ptr_dst = (u08 *)image->GetBits();
	u08 *ptr_src = data;

	/* now copy the contents from the unsigned char data array to the CImage array */
	for (j = 0; j < height; j++) {
		for (i = 0; i < width; i++) {
			*(ptr_dst++) = *(ptr_src + 2);
			*(ptr_dst++) = *(ptr_src + 1);
			*(ptr_dst++) = *(ptr_src);

			ptr_src += 3;
		}
		ptr_dst += pitch - (width * 3); 
	}

	return;
}

/* save full-size image to a file */
void Image :: saveImage(char *name) {
	CImage image;

	/* create a CImage object so that we can use it to save the data as a valid image file */
	image.Create(width, height, 24, 0);

	if (image.IsNull()) {
		fprintf(stderr,"ERROR! Unable to create image %s!\n", name);
		fflush(stderr);
		exit(-1);
	}

	/* copy the data from the unsigned char array */
	copyBackImageData(&image);

	image.Save(name);

	return;
}



/* used to save extremely large images (like the final photomosaic), because CImage has
   a max size of only a few million pixels */
void Image :: saveToBMP(char *name) {
	int i, j;
	FILE *outfile;
	char final_name[512];
	u08 *data_ptr;
	int size_of_file_header = 14;
	int size_of_info_header = 40;
	int size_of_file;
	int zero = 0;
	char *ptr;

	/* append a bmp to the name to make sure that it's a bmp */
	sprintf_s(final_name, "%s.bmp", name);
	fopen_s(&outfile, final_name, "wb");

	// bitmapfileheader
	size_of_file = size_of_file_header + size_of_info_header 
									   + 3 * width * height + 2;	// 3 represents 24-bit/pix

	fprintf(outfile,"%c%c",(char)0x42, (char)0x4d);
	ptr = (char*)&size_of_file;
	fprintf(outfile,"%c%c%c%c",*(ptr), *(ptr+1), *(ptr+2), *(ptr+3));
	fprintf(outfile,"%c%c%c%c",(char)0x00, (char)0x00, (char)0x00, (char)0x00);
	fprintf(outfile,"%c%c%c%c",(char)0x36, (char)0x00, (char)0x00, (char)0x00);


	// bitmapinfoheader
	ptr = (char*)&size_of_info_header;
	fprintf(outfile,"%c%c%c%c",*(ptr), *(ptr+1), *(ptr+2), *(ptr+3));				// biSize

	ptr = (char*)&width;
	fprintf(outfile,"%c%c%c%c",*(ptr), *(ptr+1), *(ptr+2), *(ptr+3));							// biWidth

	ptr = (char*)&height;
	fprintf(outfile,"%c%c%c%c",*(ptr), *(ptr+1), *(ptr+2), *(ptr+3));							// biHeight
	fprintf(outfile,"%c%c",(char)0x01, (char)0x00);			// biPlanes
	fprintf(outfile,"%c%c",(char)0x18, (char)0x00);			// biBitCount

	ptr = (char*)&zero;
	fprintf(outfile,"%c%c%c%c",*(ptr), *(ptr+1), *(ptr+2), *(ptr+3));								// biCompression
	fprintf(outfile,"%c%c%c%c",*(ptr), *(ptr+1), *(ptr+2), *(ptr+3));								// biSizeImage = 0
	fprintf(outfile,"%c%c%c%c", (char)0x12, (char)0x0b, (char)0x00, (char)0x00);		// biXPelsPerMeter
	fprintf(outfile,"%c%c%c%c", (char)0x12, (char)0x0b, (char)0x00, (char)0x00);		// biYPelsPerMeter
	fprintf(outfile,"%c%c%c%c",*(ptr), *(ptr+1), *(ptr+2), *(ptr+3));								// biClrUsed
	fprintf(outfile,"%c%c%c%c",*(ptr), *(ptr+1), *(ptr+2), *(ptr+3));								// biClrImportant

	/* fill in the data upside down so start from the bottom */
	data_ptr = data + ((height - 1) * width) * 3;
	for (j = 0; j < height; j++) {
		for (i = 0; i< width; i++) {
			fprintf(outfile,"%c", *(data_ptr+2));
			fprintf(outfile,"%c", *(data_ptr+1));
			fprintf(outfile,"%c", *(data_ptr));
			data_ptr += 3;
		}

		data_ptr = data_ptr - (width * 6);		
	}

	fprintf(outfile,"%c%c",(char)0x00, (char)0x00);

	fclose(outfile);

	return;
}

void Image::copyImage(Image* oriImg, Image* newImg)
{
	height = oriImg->height;
	width = oriImg->width;
	newImg = new Image(width,height);
    for(int i=0;i<height;i++)
		for(int j=0;j<width;j++)
		{
			*(newImg->data + 3*(i*width+j)  ) = *(oriImg->data + 3*(i*width+j)  );
			*(newImg->data + 3*(i*width+j)+1) = *(oriImg->data + 3*(i*width+j)+1);
			*(newImg->data + 3*(i*width+j)+2) = *(oriImg->data + 3*(i*width+j)+2);
		}
	return;

}