
/*
    pbrt source code Copyright(c) 1998-2010 Matt Pharr and Greg Humphreys.

    This file is part of pbrt.

    pbrt is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.  Note that the text contents of
    the book "Physically Based Rendering" are *not* licensed under the
    GNU GPL.

    pbrt is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

 */

#if defined(_MSC_VER)
#pragma once
#endif

#ifndef PBRT_CORE_RNG_H
#define PBRT_CORE_RNG_H

#include <ctime>

// core/rng.h*
#include "pbrt.h"
#include "probes.h"

// Random Number Declarations
class RNG {
public:
    RNG(uint32_t seed = 5489UL) {
        mti = N+1; /* mti==N+1 means mt[N] is not initialized */
		Seed(seed); 
    }

    void Seed(uint32_t seed) const;
    float RandomFloat() const;
    unsigned long RandomUInt() const;
	unsigned long leiRandomUInt() const; // Written by Lei Xiao

private:
    static const int N = 624;
    mutable unsigned long mt[N]; /* the array for the state vector  */
    mutable int mti;
};



#endif // PBRT_CORE_RNG_H
